
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.024.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "install OS packages and determine if CPAN modules are packaged for the OS ",
  "dist_author" => [
    "Ian Kluft <cpan-dev\@iankluft.com>"
  ],
  "dist_name" => "Sys-OsPackage",
  "dist_version" => "0.1.4",
  "license" => "artistic_2",
  "module_name" => "Sys::OsPackage",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Dumper" => 0,
    "Perl::PrereqScanner::NotQuiteLite" => 0,
    "Sys::OsRelease" => "v0.2.3",
    "Term::ANSIColor" => 0,
    "autodie" => 0,
    "base" => 0,
    "perl" => "5.006",
    "strict" => 0,
    "utf8" => 0,
    "warnings" => 0
  },
  "script_files" => [
    "bin/fetch-reqs.pl"
  ],
  "test_requires" => {
    "Cwd" => 0,
    "File::Copy::Recursive" => 0,
    "Getopt::Long" => 0,
    "Test::More" => 0
  }
);


my %fallback_build_requires = (
  "Cwd" => 0,
  "File::Copy::Recursive" => 0,
  "Getopt::Long" => 0,
  "Module::Build" => "0.28",
  "Test::More" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
